#!/usr/bin/perl
<<DOC;
-------------------------------------------------------------------------------
Nom : ANDREA F. MONACO

BAO2.pl -- nettoie et extrait le contenu textuel des balises <title> et 
<description> des fichiers XML-RSS en entrée et produit trois sorties: 

	- un fichier texte brut
	- un fichier XML étiqueté en POS (TreeTagger).
	- un fichier .TAL étiqueté en dépendances syntaxiques (Talismane).

nom_rubrique : 'france', 'media', 'international'
Mode d'emploi : perl BAO2.pl chemin_corpus nom_rubrique (ex.: france)
-------------------------------------------------------------------------------
DOC
use XML::RSS;
use strict;

use utf8;
binmode STDOUT, ":utf8";

my $folder="$ARGV[0]";
my $rubrique="$ARGV[1]";
# -----------------------------------------------------------------------------
# PREPARING ARGUMENTS
my $n_rubrique = $rubrique;
my %rub_conv = ("france"=>"3224", "media"=>"3236", "international"=>"3210");
my $rubrique = $rub_conv{"$rubrique"};
# on s'assure que le répertoire ne se termine pas par "/"
$folder=~ s/[\/]$//;
# -----------------------------------------------------------------------------
my %doublons;
my $i=1;
# -----------------------------------------------------------------------------
# OPENING FILES FOR WRITING
open my $txt, ">:encoding(UTF-8)", "./sorties/txt/BAO2-$folder-$n_rubrique.txt";
open my $xml, ">:encoding(UTF-8)", "./sorties/xml/BAO2-$folder-$n_rubrique.xml";
open my $talismane, ">:encoding(UTF-8)", "./sorties/tagged/BAO2-$folder-$n_rubrique.txt";
# -----------------------------------------------------------------------------
# PRINTING XML HEADERS
print $xml "<?xml version='1.0' encoding='UTF-8'?>\n";
print $xml "<file>\n";
print $xml "\t<name>Andrea F. Monaco</name>\n";
# -----------------------------------------------------------------------------
# RETRIEVING AND PROCESSING XML FILES
&finding_files($folder);
# -----------------------------------------------------------------------------
# CLOSING FILES
print $xml "</file>\n";
close $xml;
close $txt;
close $talismane;
exit;
# -----------------------------------------------------------------------------
# RETRIEVING AND PROCESSING XML FILES
sub finding_files {
	# annee : /2018 ($folder)
    my $path = shift(@_);
    # on récupère la liste des fichiers de $folder
    opendir(DIR, $path) or die "can't open $path: $!\n";
    my @files = readdir(DIR);
    closedir(DIR);
    # on ordonne la liste de fichiers
    foreach my $file (sort { $a <=> $b } @files) {
		next if $file =~ /^\.\.?$/;
		$file = $path."/".$file;
		# /mois/jour/horaire...
		if (-d $file) {
		    print "$file\t↵\n";
			&finding_files($file);	#recurse!
		}
		# ...les fichiers ont été trouvé !
		if (-f $file) {
		    if ($file =~ /$rubrique.+\.xml$/) {
		    	print $xml "\t<fileid>$file</fileid>\n";	
				print $i++," : $file \n";
				# --------------------------------------------------------------
				# XML::RSS VERSION (BAO1)
				my $rss=new XML::RSS;
				eval {$rss->parsefile($file); };
				if( $@ ) {
					$@ =~ s/at \/.*?$//s;
					print STDERR "\nERROR in '$file':\n$@\n";
				} 
				else {
					my $count = 1;
					# le fichier a été parsé, on récupère les données
				 	my $date = $rss->{'channel'}->{'pubDate'};
				 	print $xml "\t<pubDate>$date</pubDate>\n";
				 	my $talismane_title = "";
				 	my $talismane_description = "";
			     	foreach my $item (@{$rss->{'items'}}) {
			     		# pour chaque item on extrait les infos de <title> et <description>
						my $description = $item->{'description'};
						my $title = $item->{'title'};
						my($title, $description) = &cleaning($title, $description);
						if (exists $doublons{$title}) {
							$doublons{$title}++;
						}
						else {
							$doublons{$title}=1;
							# --------------------------------------------------------------
							# BAO2			
							$title =~ s/ +$//g;
							$description =~ s/ +$//g;			
							my $title_tal = $title;
							my $description_tal = $description;
							$title_tal =~ s/([…\.\?\!]+)/$1\n\n/g;
							$title_tal =~ s/(\n)+ +/$1/g; 
							$description_tal =~ s/([…\.\?\!]+)/$1\n\n/g;
							$description_tal =~ s/(\n)+ +/$1/g;
							$talismane_title = $talismane_title."££debuttitre££\n\n".$title_tal."§\n\n££fintitre££\n\n" ;
							$talismane_description = $talismane_description."££debutdescription££\n\n".$description_tal."§\n\n££findescription££\n\n" ;
							print $txt "#$title\n";
							print $txt "$description\n\n";
							print $xml "\t<item n='$count'>\n" ;
							my ($TTtitle, $TTdescription) = &XMLTTtagging($title, $description);
							print $xml "\t\t<title>$TTtitle</title>\n" ;
							print $xml "\t\t<description>$TTdescription</description>\n";
							print $xml "\t</item>\n";
							$count++;
						}
					}
					print $talismane "## titre : ".$file."\n";
					my $tagged_talismane = &etiquettage_talismane($talismane_title);
					print $talismane "$tagged_talismane";
					print $talismane "## description : ".$file."\n";
					my $tagged_talismane = &etiquettage_talismane($talismane_description);
					print $talismane "$tagged_talismane";
				}		
			}
		}
    }
} # finding_files
# -----------------------------------------------------------------------------
# CLEANING <TITLE> AND <DESCRIPTION TEXT CONTENT
sub cleaning {

	my ($title, $description) = @_;

	# cleaning <title>
	$title .= "." ;
	$title =~ s/ +\././g;
	$title =~ s/<[^>]+>//g;
	$title =~ s/&amp;/&/g;
	$title =~ s/&lt;/</g;
	$title =~ s/&gt;/>/g;
	$title =~ s/&quot;/"/g;
	$title =~ s/&apos;/'/g;
	$title =~ s/’/'/g;

	# cleaning <description>
	$description =~ s/\n//g;
	$description =~ s/&#39;/'/g;
	$description =~ s/&#34;/"/g;
	$description =~ s/&#38;#39;/'/g;
	$description =~ s/&#38;#34;/"/g;
	$description =~ s/&amp;/&/g;
	$description =~ s/&lt;/</g;
	$description =~ s/&gt;/>/g;
	$description =~ s/&quot;/"/g;
	$description =~ s/&apos;/'/g;
	$description =~ s/<[^>]+>//g;
	$description =~ s/’/'/g;
	$description =~ s/”/"/g;
	$description =~ s/“/"/g;

	return $title, $description;

} # cleaning

sub XMLTTtagging {

	my ($title, $description) = @_;

	open my $tmp, ">:encoding(utf8)", "./tmp/tmp.txt";
	print $tmp $title;
	close $tmp;

	system("perl ./treetagger/tokenise-utf8.pl -f ./tmp/tmp.txt | tree-tagger -quiet -token -lemma -no-unknown ./treetagger/french-utf8.par > ./tmp/temp-tagged-text.txt");
	system("perl ./treetagger/treetagger2xml-utf8.pl ./tmp/temp-tagged-text.txt utf8");

	local $/=undef;
	open my $tagged, "<:encoding(utf8)", "./tmp/temp-tagged-text.txt.xml";
	my $tagged_title = <$tagged>;

	close $tmp;

	$tagged_title =~ s/<\?xml.+?>//;

	#----------------------------------------------------

	open my $tmp, ">:encoding(UTF-8)", "./tmp/tmp.txt";
	print $tmp $description;
	close $tmp;

	system("perl ./treetagger/tokenise-utf8.pl -f ./tmp/tmp.txt | tree-tagger -quiet -token -lemma -no-unknown ./treetagger/french-utf8.par > ./tmp/temp-tagged-text.txt");
	system("perl ./treetagger/treetagger2xml-utf8.pl ./tmp/temp-tagged-text.txt utf8");

	open my $tagged, "<:encoding(UTF-8)", "./tmp/temp-tagged-text.txt.xml";
	my $tagged_description = <$tagged>;

	close $tmp;

	$tagged_description =~ s/<\?xml.+?>//;

	return $tagged_title, $tagged_description;

} # XMLTTtagging

sub etiquettage_talismane {

	my $var = shift @_;
	open my $tmp, ">:encoding(UTF-8)", "./tmp/bao1_test.txt";
	print $tmp $var;
	close $tmp;
	system("java -Xmx1G -Dconfig.file=talismane/talismane-fr-5.0.4.conf -jar talismane/talismane-core-5.1.2.jar --analyse --sessionId=fr --encoding=UTF8 --inFile=./tmp/bao1_test.txt --outFile=./tmp/bao1_test.tal");
	my $lefil;

	local $/=undef;
	open my $tagged, "<:encoding(UTF-8)", "./tmp/bao1_test.tal";
	$lefil = <$tagged>;
	close $tagged;

	return $lefil;
}
